#include <stdio.h>
#include <string.h>
#include <math.h>
#include "ant.h"
#include "event.h"
#include "area.h"
#include "websocket/wsTimerThread.h"
#include "sys_setting.h"
#include <numeric>

int site::index()const
{
	//return m_algo+(m_num_dims<<1);
    return (m_algo<<4) + m_num_dims;
}

site::site(int id)
	:m_algo(0)
	,m_num_dims(0)
	,m_id(id)
	,m_path_empty(true)
{
	//m_time = time(0);
	for(uint32_t i=0;i<m_timeoff_count.size();i++)
		m_timeoff_count[i]=0;
}

const algo_config&site::config()const
{
	return g_config[index()];
}

void ant::set_path(const std::vector<line_v>& v_line, std::vector<line_v>::const_iterator itm)
{
	auto it = itm;
	for(int i = 0; i < 2 && it != v_line.end(); ++it,++i)
		m_path[0][i]=*it;

	it=itm-1;
	for(int i=0;i<2 && it>=v_line.begin();--it,++i)
	{
		m_path[1][i]=*it;
		m_path[1][i].swap_point();
	}

    for(int i=0;i<2;i++)
    {
       point _p=m_path[i][0].line::projection(*this);
       m_path[i][0].set_point(0,_p);
    }

	for(const auto&p:m_path)
	{
		log_info("site-path: %s",p.to_str().c_str());
	}
}

void ant::set_path(const std::vector<line_v>&v_line_)
{
	std::vector<line_v> vl(v_line_);
	vl.reserve(vl.size()+2);
	//找到距离天线最近的端点
	auto min_it = vl.begin();
	double dist = 10;
	for(auto it = vl.begin(); it != vl.end(); it++)
	{
		double d = it->as_line().dist(*this);
		if(d < dist)
		{
			dist = d;
			min_it = it;
		}
	}

	if(min_it==vl.end())
	{
		log_error("分站路径距离分站太远site_id=%d",m_id);
		return;
	}

	if(abs(min_it->v[0].dist(*this) - dist) < 1)
	{
		set_path(vl, min_it);
	}
	else if(abs(min_it->v[1].dist(*this) - dist) < 1)
	{
		set_path(vl, min_it+1);
	}
	else
	{
		point proj=min_it->projection(*this);
		vl.insert(min_it+1,line_v(proj,min_it->v[1]));
		min_it->set_point(1,proj);

		if(min_it->v[0].z)//slope ..555
		{
			double slope=min_it->v[0].z;
			double len_a=min_it->v[0].dist((min_it+1)->v[1]);
			double len_0=slope*min_it->length()/len_a;
			double len_1=slope-len_0;
			
			min_it->v[0].z=min_it->v[1].z=len_0;
			(min_it+1)->v[0].z=(min_it+1)->v[1].z=len_1;
		}

		set_path(vl,min_it+1);
	}
}

/*
 * @brief			天线相位是否异常
 * @param			无
 * @return			相位异常返回true，否则返回false
 * @note
 * @warning
 * @bug
 * */
bool ant::phase_error()
{
	if(m_cache_poa.size()<5){
		return false;
	}

	double sum = std::accumulate(m_cache_poa.begin(), m_cache_poa.end(), 0.0);

	return (sum < 1E-4?true:false);

}

void site::set_path(const std::vector<line_v>&v_line)
{
	if(v_line.empty())
		return;

	const auto&find_line=[](const point&pt,int first,std::vector<line_v>&vl){
		for(auto it=vl.begin();it!=vl.end();it++)
		{
			if(it->v[first].dist(pt)<1)
				return it;
			if(it->v[first?0:1].dist(pt)<1)
			{
				point p=it->v[0];
				it->v[0]=it->v[1];
				it->v[1]=p;
				return it;
			}
		}

		return vl.end();
	};
	
	//构造一个首尾相连的结构
	std::vector<line_v> vl(v_line.begin()+1,v_line.end());
	std::vector<line_v> target(v_line.begin(),v_line.begin()+1);
#if 0
	target[0][0].z=target[0][1].z=slope[0];
	for(int i=0,c=vl.size();i<c;i++)
		vl[i][0].z=vl[i][1].z=slope[i+1];
#endif

	for(;;)
	{
		auto it=find_line(target.back().v[1],0,vl);
		if(it==vl.end())
			break;

		target.insert(target.end(),it,it+1);
		vl.erase(it);
	}

	for(;;)
	{
		auto it=find_line(target.front().v[0],1,vl);
		if(it==vl.end())
			break;

		target.insert(target.begin(),it,it+1);
		vl.erase(it);
	}

	log_info("初始化分站路径:site_id=%d",m_id);

	for(int i=0,c=target.size();i<c;i++)
	{
		log_info("path-id=%d,line=%s",i,target[i].to_string().c_str());
	}

	for(int i = 0,c = m_ant.size(); i < c; i++)
	{
		if(m_ant[i].empty())
		{
			log_error("分站未设置天线坐标 site_id=%d,ant_index=%d", m_id, i);
			continue;
		}
		m_ant[i].set_path(target);
	}
	m_path_empty= !m_ant[0][0].valid() && !m_ant[0][1].valid() ;
}

double site::get_site_dist(const int& sid)
{
    auto it_site = sit_list::instance()->get(sid);

    if(it_site){
        return this->dist(*it_site);
    }else{
        return 0.0;
    }
}

std::vector<point> ant::getsol(const double &dist) const
{
	std::vector<point> v;
	for(const auto & p : m_path)
	{
		if(!p.valid())
			continue;

		point pt;
		double d  = dist;

        //logn_info(3, "[pdoa] dist=%.2f, line.length()=%.2f, %s", dist, p.m_line[0].length(), p.to_str().c_str());
		if(dist <= p.m_line[0].length())
		{
			d += d*p.m_line[0][0].z;
			pt = point(p.m_line[0][0].x + d*p.m_line[0].cos() , p.m_line[0][0].y + d*p.m_line[0].sin());
		}
		else if(p.m_line[1].length()>0)
		{
			d -= p.m_line[0].length()*(1-d*p.m_line[0][1].z);
			d += d*p.m_line[1][0].z;

			pt = point(p.m_line[1][0].x+d*p.m_line[1].cos(),p.m_line[1][0].y+d*p.m_line[1].sin());
		}
		else
		{
			continue;
		}

		v.push_back(pt);
	}
	return std::move(v);
}

bool visit_site_status::visit(std::shared_ptr<site> s)
{
	time_t now = time(0);
	int diff = now - s->m_time;
	event_tool::instance()->handle_event(OT_DEVICE_READER, ET_READER_ERROR, s->m_id, READER_TIMEOUT, diff, diff>READER_TIMEOUT);

	// 定位基站长时间无定位告警
	/*if(s->m_loc_time > 0 && s->m_device_type_id == DEVICE_TYPE::LOCATE_SITE)
	{
		int diff = now - s->m_loc_time;
		log_info("site_no_position: site_id=%d, diff=%d, loc_time=%ld", s->m_id, diff, s->m_loc_time);
		event_tool::instance()->handle_event(OT_DEVICE_READER, ET_READER_LONG_TIME_NO_POSITION, s->m_id, CYaSetting::m_sys_setting.site_no_position_time, diff, diff>CYaSetting::m_sys_setting.site_no_position_time);
	}*/

	// 定位基站天线poa异常告警
	if(s->m_device_type_id == DEVICE_TYPE::LOCATE_SITE){
		if(s->m_ant[0].phase_error()&&s->m_ant[1].phase_error()){
			event_tool::instance()->handle_event(OT_DEVICE_READER, ET_READER_ANTENNA_PHASE_ABNORMAL, s->m_id, 0, 3, true);
		}else if(s->m_ant[0].phase_error()|| s->m_ant[1].phase_error()){
			if(s->m_ant[0].phase_error()){
				// 如果1天线相位值错误
				event_tool::instance()->handle_event(OT_DEVICE_READER, ET_READER_ANTENNA_PHASE_ABNORMAL, s->m_id, 0, 1, true);
			}else if(s->m_ant[1].phase_error()){
				// 如果2天线相位值错误
				event_tool::instance()->handle_event(OT_DEVICE_READER, ET_READER_ANTENNA_PHASE_ABNORMAL, s->m_id, 0, 2, true);
			}
		}else{
			// 如果都没错误，告警消失
			event_tool::instance()->handle_event(OT_DEVICE_READER, ET_READER_ANTENNA_PHASE_ABNORMAL, s->m_id, 0, 0, false);
		}
	}

	// 更新设备状态
	int state = (diff > READER_TIMEOUT ? 1 : 0);
	//log_info("[device_state] rid=%d, diff=%d, timeout=%d, state=%d, now=%lld, rtime=%lld", s->m_id, diff, READER_TIMEOUT, state, now, s->m_time);
	sys::device_state ds(s->m_id, s->m_device_type_id, state, now);
	swsTimerThrd.upt_device_state(ds);

	return true;
}


void site::create_area()
{
    m_area=std::make_shared<area>(-m_id,0,0,m_scale,m_map_id,1<<7);
}

void site::clear_event()
{
    event_tool::instance()->handle_event(OT_DEVICE_READER, ET_READER_ERROR, id(), READER_TIMEOUT, 0, false);
    event_tool::instance()->handle_event(OT_DEVICE_READER, ET_READER_POWER_BY_BATTERY, id(), 0, 0, false);
}

/*
   处理分站供电状态，交流供电时，ac_down=false，直流供电时,ac_down=true

   目前只有大分站实现了这个功能，并且井下安装时是否接入了该电信号也不确定
   ，所以需要有张表定义某个ID是否需要告警
 */
void site::on_power_status(bool ac_down)//电源状态
{
	if(!m_power_check_enable || ac_down == m_power_ac_down)
	{
        return;
    }
    m_power_ac_down=ac_down;
    event_tool::instance()->handle_event(OT_DEVICE_READER,ET_READER_POWER_BY_BATTERY,id(),tool_time::now_to_seconds(),ac_down,m_power_ac_down);
    log_info("[event warning: reader power supply by battery] reader_id: Power %d->%d.",id(),!m_power_ac_down,m_power_ac_down);
}

algo_config site::g_config[]=
{
	{ "tof-1",  1, 2, 0.1, 1 },
	{ "tdoa-1", 2, 2, 0.1, 1 },
	{ "tof-2",  2, 3, 0.1, 1 },
	{ "tdoa-2", 3, 3, 0.1, 1 },
	{ "tof-3",  3, 4, 0.1, 1 },
	{ "tdoa-3", 4, 4, 0.1, 1 }
};
/*
#ifdef _TEST
int main()
{
	log_init("./log.ini");
    //sit_list *sl = sit_list::instance();
	sit_list::instance()->load("data_reader_antenna.txt","path_tof.txt");
	sit_list::instance()->get(209)->solving(0,100);
	sit_list::instance()->get(209)->solving(1,100.5);
    //std_info("---%d",(*sl)[209].m_ant[0].m_path.size());
    //std_info("---%d",(*sl)[209][0].size());
    //std_info("---%s",(*sl)[209][0][0][0].to_string().c_str());
}
#endif
*/
