/**
 * 分站数据结构 site_list
 */
#ifndef _ANT_LIST_HPP_
#define _ANT_LIST_HPP_

#include <math.h>
#include <array>
#include <deque>
#include <tuple>
#include <time.h>
#include <sstream>
#include <memory>
#include <memory.h>
#include <algorithm>
#include <sstream>
#include "log.h"
#include "line.h"
#include "point.h"
#include "write-copy.h"
#include "net-service.h"
#include "common.h"
#include "tool_time.h"
#include "card_path.h"
#include "db_api/CDBSingletonDefine.h"
#include <boost/circular_buffer.hpp>

class client;
class area;
struct path
{
    std::array<line_v,2>	m_line;
    path()
    {
    }

    std::string to_str() const
    {
        std::stringstream ss;
        for(int i=0;i<2;i++)
        {
            ss<<"line:" <<m_line[i].to_string()<<"slope:"<<m_line[i][0].z<< " cos:"<<m_line[i].cos()<<" sin:"<<m_line[i].sin()<<" | ";
        }
        return ss.str();
    }
    bool valid() const
    {
        return !m_line[0].empty();
    }
    line_v & operator[](int i)
    {
        return m_line[i];
    }
    const line_v & operator[](int i) const
    {
        return m_line[i];
    }
};
//?
struct algo_config
{
    const char*desc;
    int min_msg_cnt;
    int best_msg_cnt;
    double min_wait_time;
    double max_wait_time;
};

// 基站覆盖范围信息
struct site_coverage_info{
	double m_distance;	// 距离
	std::string m_cid;	// 卡号
	uint64_t m_time;	// 时间

	site_coverage_info()
		: m_distance(0.0)
		, m_cid("")
		, m_time(0)
	{}
};
//
struct ant :point
{
    std::array<path,2> m_path;
    int m_id;
    double m_angle = 0;
	boost::circular_buffer<double> m_cache_poa;		// 缓存5个天线的相位差值

	ant()
	{
		m_cache_poa.set_capacity(5);
	}

    path & operator[](int i)
    {
        return m_path[i];
    }

    const path & operator[](int i) const
    {
        return m_path[i];
    }

    size_t size() const
    {
        return m_path.size();
    }

	void set_path(const std::vector<line_v>&v_line);
	void set_path(const std::vector<line_v>&v_line,std::vector<line_v>::const_iterator it);
    std::vector<point> getsol(const double &dist) const;

	bool phase_error();
};

struct site:point,std::enable_shared_from_this<site>
{
    static algo_config g_config[];
    int      m_algo;			// TOF:0, TDOA:1, PDOA:2
    int      m_num_dims;	    // 1维:1, 2维:2, 3维:3
    double  m_scale = 2.0;  // 地图比例尺
    point    m_position;
    mutable double m_height=1.5;
    std::array<ant,2> m_ant;
    mutable double m_ant_dist=0;
    mutable double m_ant_dist_sum_new=0;
    mutable int m_ant_dist_cnt_new=0;
    ///分站位置 READER_TYPE_ID, 井上分站，井下分站
    int m_reader_type_id = 0;
    int m_map_id = 0;
    int m_area_id = 0;
    // 设备类型，分站、通信分站、交通灯等
    int m_device_type_id=0;
	// 是否为网络设备，0为can设备，1为网络设备
	int m_net_device_status;
    
    //pdoa
    double m_pdoa_offset = 0.0;     // pdoa分站天线相位偏移量,90度的偏移值
    int m_pdoa_direction = 0;       // pdoa分站天线1朝向
    std::shared_ptr<client> m_clt = nullptr;
    std::shared_ptr<area> m_area = nullptr;

	time_t m_time;
	time_t m_loc_time = time(0);	// 基站定位时间
//  time_t m_rec_time;              // 接收数据的时间
//  time_t m_lost_time;		        // 丢失时间
//  time_t m_last_send_time;	    // 最后向前端发送时间

    int m_id{-1};
//  int  m_tick_count;			    // 分站发生消息的计数器

//  unsigned char m_reader_dir;		// 分站方向，对于大小分站适用
//  unsigned char m_relay_counts;		// 大小分站经过中继数
    bool m_power_check_enable=false;//该分站是否启用交流掉电检测功能(告警功能)
    bool m_power_ac_down=false;     //false=交流电供电状态
    bool m_path_empty;

	std::array<int,32>	m_timeoff_count;
	std::array<site_coverage_info, 2> m_coverage;	// 0为正向参数，1为负向参数
	int  m_package_count=0;
    int  m_special = 0;
	int  m_down_stream_idx = 0;

    site(int id=-1);

	time_t m_site_time=0;		//分站报文中的时间
	void set_site_time(time_t site_time)
	{
		m_site_time=site_time;
	}

	time_t last_site_time()const
	{
		return m_site_time;
	}

    int index()const;
    const algo_config&config()const;
    int id()const
    {
        return m_id;
	}
    void set_client(const std::shared_ptr<client>& clt)
    {
		if(m_clt != clt)
			m_clt = clt;
		m_time = time(0);
    }

    std::shared_ptr<client> get_client()
    {
        return m_clt;
    }

    bool is_up_site()
    {
        return READER_TYPE_ID_UP == m_reader_type_id;
    }
    bool is_abnormal_site()
    {
        bool f=false;
        double d=ant_dist();
        if(!is_up_site() &&(d<1e-5 && d>-1e-5) )
            f=true;
        return f;
    }
    void create_area();
    const std::shared_ptr<area> &get_area()const{return m_area;}
    void clear_event();
    point get_dstp(const point pt) const
    {
        point tmp;
        for(const auto & p : m_ant[0].m_path){
            for(int i=0;i<2;i++){
                if(!p[i].empty()){
                    if(p[i].contain(pt,0.01)){
                        tmp = p[i][0];
                    }
                }
            }
        }
        if(tmp.empty())
            log_info("(%f,%f) can't find the point at [%d]",pt.x,pt.y,m_id);
        return tmp;
    }

    void count_ant_dist(double dist_tof1, double dist_tof2)const
    {
        if(dist_tof1<10 || dist_tof2<10)
            return;
        double dist = fabs(dist_tof1 - dist_tof2);
        if(dist>5)
            return;
        m_ant_dist_sum_new += dist;
        m_ant_dist_cnt_new++;

        if(m_ant_dist_cnt_new >= 2500)
        {
            m_ant_dist = m_ant_dist_sum_new / m_ant_dist_cnt_new;
            m_ant_dist_sum_new = 0;
            m_ant_dist_cnt_new = 0;
        }
    }

    double ant_dist()const
    {
        return m_ant[0].dist(m_ant[1]);
    }

    bool is_path_empty()const
    {
        return m_path_empty;
    }

    bool have_valid_path()const
    {
        return m_id != -1 && ant_dist() > 0.1;
    }

    std::string to_string()const
    {
        std::stringstream ss;
        ss<<"site_id:"<<m_id<<"x:"<<x<<" y: "<<y<<" scale:"<<m_scale;
        for(const auto&a:m_ant)
        {
            ss<<"<";
            for(const auto&p:a.m_path)
            {
                ss<<"{"<<p.to_str()<<"}";
            }
            ss<<">";
        }
        return ss.str();
    }

    const point&path(int i)const
    {
        static point p;
        if(i>=(int)m_ant[0].m_path.size())
            return p ;
        return m_ant[0].m_path[i].m_line[0][1];
    }

	void set_path(const std::vector<line_v>&v_line);
    std::vector<point> solving(int ant_id, double dist)const
    {
        const ant &a = m_ant[ant_id];
        if(dist < 50 && dist > 0)
        {
            if(dist < m_height)
            {
                m_height = dist;
                dist = 0;
            }
            else
            {
                dist = sqrt(dist*dist - m_height*m_height);
            }
        }
        return std::move(a.getsol(dist));
    }

    std::vector<point> solving_pdoa(const int& ant_id, double dist)const
    {
        const ant& a = m_ant[ant_id];
        //logn_info(3, "[pdoa] solving_pdoa, dist=%.4f, a.x=%.4f, a.y=%.4f", dist, a.x, a.y);
        return std::move(a.getsol(dist));
    }

    ant&operator[](int i)
    {
        return m_ant[i];
    }

    const ant&operator[](int i) const
    {
        return m_ant[i];
    }

    void set_ex()
    {
        if(-1 == m_id)
        {
            return;
        }

        if(m_ant[0]==m_ant[1])
        {
            log_warn("%d分站天线坐标相等.", m_id);
        }

        set( (m_ant[0].x+m_ant[1].x)/2,(m_ant[0].y+m_ant[1].y)/2);
    }

    void delete_antenna(int antidx)
    {
        m_ant[antidx].set(point(0,0));
    }

/*

	处理分站供电状态，交流供电时，ac_down=false，直流供电时,ac_down=true

	目前只有大分站实现了这个功能，并且井下安装时是否接入了该电信号也不确定
	，所以需要有张表定义某个ID是否需要告警

*/

	void on_power_status(bool ac_down);     //电源状态
    void set_algo(int _algo)
    {
        m_algo = _algo;
    }

    double get_site_dist(const int& sid);

	void push_poa(const double& poa1, const double& poa2)
	{
		m_ant[0].m_cache_poa.push_back(poa1);
		m_ant[1].m_cache_poa.push_back(poa2);
	}
};

struct visit_site_status:visitor<std::shared_ptr<site>>
{
	bool visit(std::shared_ptr<site> s);
};

class sit_list_type_staff {};
class sit_list_type_vehicle {};

template <class sit_list_type>
struct sit_list_template:single_base<sit_list_template<sit_list_type>, int, std::shared_ptr<site>>
{
public:
    void load(const std::string &id)
    {
        int sid=-1;
        if(!id.empty())sid=std::stoi(id);
        read_sit_list(sid);
        read_ant_path(sid);
    }
    ///id=-1为初始化所有
    void load_from_db(const std::string&ids="")
    {
        init_site(ids);
        load(ids);
    }
    void read_sit_list(int id=-1);
    void read_ant_path(int id=-1);
    void init_site(const std::string &ids="");
private:
	sit_list_type m_type;
};

template <class sit_list_type>
void sit_list_template<sit_list_type>::read_sit_list(int id/*=-1*/)
{
	std::string sql;
	if (std::string(typeid(m_type).name()) == "sit_list_type_staff")
	{
		sql = "SELECT antenna_id, a.reader_id, idx, a.x, a.y, a.z, a.angle \
            FROM dat_antenna a, dat_reader r \
            WHERE a.reader_id = r.reader_id";
	}
	else
	{
		sql = "SELECT antenna_id, a.reader_id, idx, a.x, a.y, a.z, a.angle \
            FROM dat_antenna_v a, dat_reader_v r \
            WHERE a.reader_id = r.reader_id";
	}

	if (-1 == id)
	{
		sql.append(";");
	}
	else
	{
		sql.append(" AND antenna_id=");
		sql.append(std::to_string(id));
		sql.append(";");

		log_info("基础数据 增加或修改天线 sql=%s", sql.c_str());
	}

	std::string Error;
	YADB::CDBResultSet DBRes;
	sDBConnPool.Query(sql.c_str(), DBRes, Error);
	int nCount = DBRes.GetRecordCount(Error);
	if (nCount < 1)
	{
		log_error("基础数据 增加或修改失败，数据库中找不到: 天线id=%d", id);
		return;
	}

	log_info("init_antenna. The record count=%ld\n", nCount);
	while (DBRes.GetNextRecod(Error))
	{
		int antenna_id = 0;
		DBRes.GetField("antenna_id", antenna_id, Error);

		int reader_id = 0;
		DBRes.GetField("reader_id", reader_id, Error);

		int idx = 0;
		DBRes.GetField("idx", idx, Error);

		int antid = idx - 1;
		if (antid >= 2 || antid < 0)
			continue;

		double x = 0;
		DBRes.GetField("x", x, Error);

		double y = 0;
		DBRes.GetField("y", y, Error);

		double z = 0;
		DBRes.GetField("z", z, Error);

		double _angle = 0;
		DBRes.GetField("angle", _angle, Error);

		log_info("ant-position:reader=%d, antid=%d, x=%.2lf, y=%.2lf, angle=%.2f", reader_id, antid, x, y, _angle);

		if (-1 == id)
		{
			std::shared_ptr<site> site_ptr = nullptr;
			site_ptr = sit_list_template<sit_list_type>::instance()->get(reader_id);
			if (!site_ptr)
				continue;
			site_ptr->m_ant[antid].m_id = antenna_id;
			site_ptr->m_ant[antid].set(x, y);
			site_ptr->m_ant[antid].m_angle = _angle;
			site_ptr->set_ex();
		}
		else
		{
			auto site_ptr = sit_list_template<sit_list_type>::instance()->get(reader_id);
			if (site_ptr)
			{
				site_ptr->m_ant[antid].set(x, y);
				site_ptr->m_ant[antid].m_angle = _angle;
				site_ptr->set_ex();
			}
			log_info("基础数据 增加或修改天线成功：天线id:%d,分站id:%d", id, reader_id);
		}
	}
}

template <class sit_list_type>
void sit_list_template<sit_list_type>::read_ant_path(int id/*=-1*/)
{
	std::string sql;
	if (std::string(typeid(m_type).name()) == "sit_list_type_staff")
	{
		sql = "SELECT reader_id,tof_flag,b_x,b_y,b_z,e_x,e_y,e_z,spacing_ratio FROM dat_reader_path_tof_n";
	}
	else
	{
		sql = "SELECT reader_id,tof_flag,b_x,b_y,b_z,e_x,e_y,e_z,spacing_ratio FROM dat_reader_path_tof_n_v";
	}

	std::map<int, std::vector<line_v>> map_path;

	if (-1 == id)
	{
		sql.append(";");
	}
	else
	{
		sql.append(" where reader_id=");
		sql.append(std::to_string(id));
		sql.append(";");

		std_debug("修改path sql=%s", sql.c_str());
		log_info("修改path sql=%s", sql.c_str());
	}

	std::string Error;
	YADB::CDBResultSet DBRes;
	sDBConnPool.Query(sql.c_str(), DBRes, Error);
	int nCount = DBRes.GetRecordCount(Error);
	if (nCount < 1)
	{
		log_error("修改path失败，数据库中找不到(dat_reader_path_tof_n): 分站id=%d", id);
		return;
	}

	log_info("read_ant_path. The record count=%ld\n", nCount);
	while (DBRes.GetNextRecod(Error))
	{
		int reader_id = 0;
		DBRes.GetField("reader_id", reader_id, Error);

		auto site_ptr = sit_list_template<sit_list_type>::instance()->get(reader_id);
		if (nullptr == site_ptr)
		{
			log_error("[site] 定义了分站路径，但是没有定义分站:%d", reader_id);
			continue;
		}
		else if (site_ptr->m_num_dims != 1) {
			log_info("[site] 多维定位分站，无需定义路径:%d", reader_id);
			continue;
		}

		int pid = 0;
		DBRes.GetField("tof_flag", pid, Error);

		double b_x = 0;
		DBRes.GetField("b_x", b_x, Error);

		double b_y = 0;
		DBRes.GetField("b_y", b_y, Error);

		double b_z = 0;
		DBRes.GetField("b_z", b_z, Error);

		double e_x = 0;
		DBRes.GetField("e_x", e_x, Error);

		double e_y = 0;
		DBRes.GetField("e_y", e_y, Error);

		double e_z = 0;
		DBRes.GetField("e_z", e_z, Error);

		double spacing_ratio = 0;
		DBRes.GetField("spacing_ratio", spacing_ratio, Error);

		log_info("src-path:site=%d,x0=%.2f,y0=%.2f,x1=%.2f,y1=%.2f", reader_id, b_x, b_y, e_x, e_y);

		point p1(b_x, b_y, spacing_ratio);
		point p2(e_x, e_y, spacing_ratio);

		map_path.insert(std::make_pair(reader_id, std::vector<line_v>()));
		map_path.find(reader_id)->second.push_back(line_v(p1, p2));
#if 0

		auto &sit_ = *site_ptr;
		if (-1 != id)//清空path
		{
			sit_.clear_path();
			log_info("修改path 清空path，分站id=%d\n", id);
		}

		if (pid == 0)
		{
			line_v l(p1, p2);
			{
				point px = l.line::projection(sit_);
				sit_.set(px);
				for (int i = 0; i < 2; i++)
				{
					path p;
					p.m_slope[0] = spacing_ratio;
					p.m_line[0] = line_v(px, l[i]);
					sit_.m_ant[i].m_path.push_back(p);
				}
			}
		}
		else
		{
			ant &a = pid < 0 ? sit_.m_ant[0] : sit_.m_ant[1];
			if (a.m_path.size() != 0)
			{
				path &p = a.m_path[0];
				p.m_line[abs(pid) - 1] = line_v(p1, p2);
				p.m_slope[abs(pid) - 1] = spacing_ratio;
			}
			else
			{
				path p;
				p.m_line[abs(pid) - 1] = line_v(p1, p2);
				p.m_slope[abs(pid) - 1] = spacing_ratio;
				a.m_path.push_back(p);
			}
			if (abs(pid) == 1)
				sit_.set(p1);
		}
#endif
	}

#if 0
	if (-1 == id)
	{
		for (auto&_s : sit_list::instance()->m_map)
		{
			_s.second->deal_path();
		}
	}
	else
	{
		auto sit_ptr = sit_list::instance()->get(id);
		if (sit_ptr)
		{
			sit_ptr->deal_path();
			log_info("修改path成功，分站id=%d\n", id);
		}
	}
#endif

	for (auto&vl : map_path)
	{
		auto sit_ptr = sit_list_template<sit_list_type>::instance()->get(vl.first);
		if (!sit_ptr)
		{
			log_error("[site] 定义了分站路径，但是没有定义分站:%d", vl.first);
			continue;
		}
		else if (sit_ptr->m_num_dims > 1) {
			log_info("[site] 多维定位分站，无需定义路径:%d", vl.first);
			continue;
		}

		sit_ptr->set_path(vl.second);
		point p;
		if (!sit_ptr->m_path_empty) {
			p = (*sit_ptr)[0][0][0].line::projection(*sit_ptr);
			sit_ptr->set(p);
		}
		if (p == *sit_ptr)
			log_info("[site_path]%s", sit_ptr->to_string().c_str());
		else
			log_info("[site_path_diff](%f,%f)--%f--%s", p.x, p.y, p.dist(*sit_ptr), sit_ptr->to_string().c_str());
	}

	card_path::init();
}

template <class sit_list_type>
void sit_list_template<sit_list_type>::init_site(const std::string &ids/*=""*/)
{
	std::string sql;
	log_info("init_site_type sql=%s", std::string(typeid(m_type).name()).c_str());
	if (std::string(typeid(m_type).name()) == "sit_list_type_staff")
	{
		/*std::string sql = "SELECT reader_id, reader_type_id, dat_reader.map_id, \
		  area_id, device_type_id, dimension, dat_map.scale,need_power_alarm \
		  ,x,y, pdoa_offset, pdoa_direction, isSpecial, down_stream_idx \
		  FROM dat_reader, dat_map where \
		  dat_reader.map_id=dat_map.map_id and state=0";*/
		sql = "SELECT r.reader_id, r.reader_type_id, r.map_id, r.area_id, r.device_type_id, r.dimension, m.scale, r.need_power_alarm, r.x, r.y, \
					   r.pdoa_offset, r.pdoa_direction, r.isSpecial, r.down_stream_idx, \
					   rc.plus_dist, rc.plus_card_id, rc.plus_occur_time, rc.minus_dist, rc.minus_card_id, rc.minus_occur_time \
					   FROM dat_reader AS r \
					   LEFT JOIN dat_map AS m ON r.map_id=m.map_id \
					   LEFT JOIN his_reader_coverage AS rc ON r.reader_id = rc.reader_id \
					   WHERE r.state=0;";
	}
	else
	{
		/*std::string sql = "SELECT reader_id, reader_type_id, dat_reader_v.map_id, \
		  area_id, device_type_id, dimension, dat_map.scale,need_power_alarm \
		  ,x,y, pdoa_offset, pdoa_direction, isSpecial, down_stream_idx \
		  FROM dat_reader_v, dat_map where \
		  dat_reader_v.map_id=dat_map.map_id and state=0";*/
		sql = "SELECT r.reader_id, r.reader_type_id, r.map_id, r.area_id, r.device_type_id, r.dimension, m.scale, r.need_power_alarm, r.x, r.y, \
					   r.pdoa_offset, r.pdoa_direction, r.isSpecial, r.down_stream_idx, \
					   rc.plus_dist, rc.plus_card_id, rc.plus_occur_time, rc.minus_dist, rc.minus_card_id, rc.minus_occur_time \
					   FROM dat_reader_v AS r \
					   LEFT JOIN dat_map AS m ON r.map_id=m.map_id \
					   LEFT JOIN his_reader_coverage AS rc ON r.reader_id = rc.reader_id \
					   WHERE r.state=0;";
	}

	if (ids.empty())
	{
		sql.append(";");
	}
	else
	{
		sql.append(" AND reader_id in (");
		sql += ids;
		sql.append(");");

		std_debug("增加或修改分站 sql=%s", sql.c_str());
		log_info("增加或修改分站 sql=%s", sql.c_str());
	}

	std::string Error;
	YADB::CDBResultSet DBRes;
	sDBConnPool.Query(sql.c_str(), DBRes, Error);
	int nCount = DBRes.GetRecordCount(Error);
	if (nCount < 1)
	{
		log_error("增加或修改失败，数据库中找不到: 分站id=%s", ids.c_str());
		return;
	}

	log_info("init_site. The record count=%ld\n", nCount);

	while (DBRes.GetNextRecod(Error))
	{
		int reader_id = 0;
		DBRes.GetField("reader_id", reader_id, Error);

		auto site_ptr = sit_list_template<sit_list_type>::instance()->get(reader_id);
		if (nullptr == site_ptr)
		{
			site_ptr = std::make_shared<site>(reader_id);
			sit_list_template<sit_list_type>::instance()->add(reader_id, site_ptr);
		}

		int reader_type_id = 0;
		DBRes.GetField("reader_type_id", reader_type_id, Error);

		int map_id = 0;
		DBRes.GetField("map_id", map_id, Error);

		int area_id = 0;
		DBRes.GetField("area_id", area_id, Error);

		int device_type_id = 0;
		DBRes.GetField("device_type_id", device_type_id, Error);

		int dimension = 0;
		DBRes.GetField("dimension", dimension, Error);

		double scale = 0;
		DBRes.GetField("scale", scale, Error);

		int power_alarm = 0;
		DBRes.GetField("need_power_alarm", power_alarm, Error);

		int down_stream_idx = 0;
		DBRes.GetField("down_stream_idx", down_stream_idx, Error);

		site_ptr->m_reader_type_id = reader_type_id;
		site_ptr->m_map_id = map_id;
		site_ptr->m_area_id = area_id;
		site_ptr->m_power_check_enable = power_alarm;
		site_ptr->m_device_type_id = device_type_id;
		site_ptr->m_num_dims = dimension;
		site_ptr->m_scale = scale;
		site_ptr->m_down_stream_idx = down_stream_idx;
		site_ptr->create_area();

		double x = 0, y = 0;
		DBRes.GetField("x", x, Error);
		DBRes.GetField("y", y, Error);
		site_ptr->x = x;
		site_ptr->y = y;

		double offset = 0.0;
		DBRes.GetField("pdoa_offset", offset, Error);
		site_ptr->m_pdoa_offset = offset;

		int direction = 0;
		if (DBRes.GetField("pdoa_direction", direction, Error)) {
			site_ptr->m_pdoa_direction = direction;
		}

		int special = 0;
		if (DBRes.GetField("isSpecial", special, Error)) {
			site_ptr->m_special = special;
		}

		double dist = 0;
		DBRes.GetField("plus_dist", dist, Error);
		site_ptr->m_coverage[0].m_distance = dist;

		std::string ctx = "";
		DBRes.GetField("plus_card_id", ctx, Error);
		site_ptr->m_coverage[0].m_cid = ctx;

		ctx.clear();
		DBRes.GetField("plus_occur_time", ctx, Error);
		site_ptr->m_coverage[0].m_time = tool_time::to_time(ctx);

		dist = 0;
		DBRes.GetField("plus_dist", dist, Error);
		site_ptr->m_coverage[1].m_distance = dist;

		ctx.clear();
		DBRes.GetField("plus_card_id", ctx, Error);
		site_ptr->m_coverage[1].m_cid = ctx;

		ctx.clear();
		DBRes.GetField("plus_occur_time", ctx, Error);
		site_ptr->m_coverage[1].m_time = tool_time::to_time(ctx);

		log_info("site_position: site=%d, x=%.2lf, y=%.2lf, area_id=%d, m_scale=%.2f, pdoa_offset=%.2f, pdoa_direction=%d, special=%d, device_type_id=%d, plus_dist=%.2f, minus_dist=%.2f", reader_id, x, y, area_id, scale, offset, direction, special, device_type_id, site_ptr->m_coverage[0].m_distance, site_ptr->m_coverage[1].m_distance);
	}
}

typedef sit_list_template<sit_list_type_staff> sit_list;
typedef sit_list_template<sit_list_type_vehicle> sit_list_v;

#endif

